////////////////////////////////////////////////////////////////
// JAKADAPTER (Dual Atari joystick converter) v.1.1 for HW v.2.1
// (C) jaka
// 1.2.2015
// Compiled using CCS 5.021, total compiled length 3239 words
////////////////////////////////////////////////////////////////
// 12.2.2014 main updated for new pins
// 18.2.2014 re-updated pin order for main, joy1task and paddle task
// 19.2.2014 updated megadrive1task and joy2task
// 11.5.2014 fixed mode2 setting in paddle2task and joy2task
//           changed megadrive 3/6 button behavior, reads controller type each time
//           decreased USB polling frequency from 1000 Hz to 500 Hz as 1000 Hz was
//           too fast for cheap chinese 6-button megadrive controller
//           Added Stelladaptor to description to make work in Stella with paddles correctly
// 1.7.2014  Changed back to 1000 Hz polling, now using timer 1 to poll megadrive controller buttons at max 500 Hz
//           Directional arrow and button B and C are still polled at 1000 Hz
// 3.7.2014  Fixed port 1 analog input settings when paddle in port 2
// 11.7.2014 Reduced acquisition time from 1 ms to 50 s when reading paddles, allows 1000 Hz USB polling. Changed
//           automatic joystick/paddle selection so that it really works in 2.0 HW
// 13.7.2014 v0.9 Added anti-jitter filtering to paddle readout
// 15.7.2014 v1.0 Added defines for all used pins
// 16.7.2014 Updated pins for HW version 2.1
// 1.2.2015  Moved pin defines to separate file
//           Patched usb.c to get rid of garbage USB string descriptors (Thanks rksjbeck from CCS forums!)
//           Changed string descriptors to 'jakadapter'. Added string 'Stelladaptor' to generic string for Linux Stella support

#include <18F24K50.h>
#device ADC=8
#build(stack=256)
#fuses PLL3X,INTRC,NOWDT,NOFCMEN,NOIESO,BROWNOUT,BORV22,NOPROTECT,NOLVP,NODEBUG,PLLEN,NOCPUDIV,PUT,MCLR
#use delay(int=16MHz, clock=48MHz, USB_FULL, act=USB)

#include <pic18_usb.h>   //Microchip PIC18Fxx5x hardware layer for usb.c
#include <usb_desc_jakadapter.h>    //USB Configuration and Device descriptors for this USB device
#include <usb.c>        //handles usb setup tokens and get descriptor reports
#include <stdlib.h>
#include "jakadap21_pindef.h"	// Pin defines for HW v2.1
#include "usb_bootloader.h"		// sets program location and reset vector for use with bootloader

#define WPUB 0xf85
#define MD_POLL_INT 47536 // 1.5 ms = 18000 clock cycle at 12 MHz -> MD_POLL_INT=65536-18000
#bit TMR1IF = 0xF9E.0	// Timer 1 interrupt flag for polling (not actually using an interrupt)

int a=0;
int b=0;
int mode; // 1 = Atari/SMS 2 = Paddles 3 = Megadrive
int mode2;

int8 out_data_1[3];	// Arrays for joy1 and joy2 USB output data
int8 out_data_2[3];
int8 md_button_poll;	// Megadrive controller multiplexed buttons are only read at 500 Hz
int16 pot1xf;	// Variables to hold paddle readout anti-jitter filter value
int16 pot1yf;
int16 pot2xf;
int16 pot2yf;

void usb_joystick_1_task(void) 
{
	#asm
	bsf	WPUB,5
	#endasm

	if (mode2==2)
		setup_adc_ports( X1_5_ANSEL|X2_9_ANSEL|X2_5_ANSEL );
	else
		setup_adc_ports( X1_5_ANSEL|X2_5_ANSEL );

	if(!input(X1_6)) // Joystick 1 button 1
		b=1;
	else
		b=0;
	if(!input(X1_9))  // Joystick 1 button 2
		b = b+2;
   
	out_data_1[0]=b; //button state goes here
	if(!input(X1_2))			// down
		out_data_1[2]=126;
	else if(!input(X1_1))		// up
		out_data_1[2]=-126;
	else
		out_data_1[2]=0;

	if(!input(X1_4))			// right
		out_data_1[1]=126;
        else if(!input(X1_3))		// left
		out_data_1[1]=-126;
	else
		out_data_1[1]=0;
	usb_put_packet(1,out_data_1,3,USB_DTS_TOGGLE);
}

void usb_paddle_task(void) 
{
	#asm
	bcf	WPUB,5
	#endasm

	if (mode2==2)
		setup_adc_ports( X1_5_ANSEL|X1_9_ANSEL|X2_9_ANSEL|X2_5_ANSEL );
	else
		setup_adc_ports( X1_5_ANSEL|X1_9_ANSEL|X2_5_ANSEL );

	int16 adc_reading=0;

	if(!input(X1_3))	// Paddle button 1
		b=1;
	else
		b=0;
	if(!input(X1_4))	// Paddle button 2
		b = b+2;
  
	out_data_1[0]=b; //button state goes here
	
	set_adc_channel(X1_9_AN);	// POT_X
	delay_us(50);
	read_adc(ADC_START_ONLY);
	while(!(adc_done())) {    
	} 
	adc_reading = read_adc(ADC_READ_ONLY);
	pot1xf = (pot1xf*3+adc_reading)/4;
	adc_reading = (28560/pot1xf);
	adc_reading = adc_reading - 112;
	if (adc_reading>255)
		adc_reading = 255;
	out_data_1[1] = (int8)(127-adc_reading);

	set_adc_channel(X1_5_AN);	// POT_Y
	delay_us(50);
	read_adc(ADC_START_ONLY);
	while(!(adc_done())) {    
	} 
	adc_reading = read_adc(ADC_READ_ONLY);
	pot1yf = (pot1yf*3+adc_reading)/4;
	if (pot1yf<10)	// Paddles have been disconnected (or bad pot contact(!))
		mode=1;
	adc_reading = (28560/pot1yf);
	adc_reading = adc_reading - 112;
	if (adc_reading>255)
		adc_reading = 255;
	out_data_1[2] = 127 - adc_reading;

	usb_put_packet(1,out_data_1,3,USB_DTS_TOGGLE);
}

void usb_megadrive_task(void) 
{
	#asm
	bsf	WPUB,5
	#endasm

	output_float(X1_7_ADD);
	out_data_1[0]=0;
	if(!input(X1_6))  // Megadrive button B
		out_data_1[0] += 2;
	if(!input(X1_9))  // Megadrive button C
		out_data_1[0] += 4;

	if(md_button_poll) {	
		output_low(X1_7); // 1st low
		delay_us(14);
		
		b=0;
		if(!input(X1_6))	// Megadrive button A
			b=1;
		if(!input(X1_9))  // Megadrive button Start
			b = b+8;
		output_high(X1_7);
		delay_us(14);
		output_low(X1_7); // 2nd low
		delay_us(15);        
		output_high(X1_7);
		delay_us(14);
		output_low(X1_7);	// 3rd low
		delay_us(14);
	
		if (input(X1_1) || input (X1_2))	// With 6-button controller 1/2/3/4 shold all be low
			mode=3;			// Set mode to 3-button megadrive controller
		else
			mode=4;
	
		output_high(X1_7);
		delay_us(13);
		if(!input(X1_1)&&mode==4)  // Megadrive button Z
			b = b+64;
		if(!input(X1_2)&&mode==4)  // Megadrive button Y
			b = b+32;
		if(!input(X1_3)&&mode==4)  // Megadrive button X
			b = b+16;
		if(!input(X1_4)&&mode==4)  // Megadrive button MODE
			b = b+128;
		mode=3;	
	
		output_low(X1_7);
		delay_us(14);        
		output_high(X1_7);
		delay_us(14);
	}		
	out_data_1[0]+=b; //button state goes here

	if(!input(X1_2))			// down
		out_data_1[2]=126;
	else if(!input(X1_1))		// up
		out_data_1[2]=-126;
	else
		out_data_1[2]=0;

	if(!input(X1_4))			// right
		out_data_1[1]=126;
    else if(!input(X1_3))		// left
		out_data_1[1]=-126;
	else
		out_data_1[1]=0;
	usb_put_packet(1,out_data_1,3,USB_DTS_TOGGLE);
}

void usb_joystick_2_task(void) 
{
	#asm
	bsf	WPUB,3
	#endasm
	if (mode==2)
		setup_adc_ports( X1_5_ANSEL|X1_9_ANSEL|X2_5_ANSEL );
	else
		setup_adc_ports( X1_5_ANSEL|X2_5_ANSEL );

	if(!input(X2_6))  // Joystick 2 button
		a=1;
	else
		a=0;

	out_data_2[0]=a; //button state goes here
	if(!input(X2_2))			// down
		out_data_2[2]=126;
	else if(!input(X2_1))		// up
		out_data_2[2]=-126;
	else
		out_data_2[2]=0;

	if(!input(X2_4))			// right
		out_data_2[1]=126;
        else if(!input(X2_3))	// left
		out_data_2[1]=-126;
	else
		out_data_2[1]=0;
		
}

void usb_paddle_2_task(void) 
{
	#asm
	bcf	WPUB,3
	#endasm
	
	if (mode==2)
		setup_adc_ports( X1_5_ANSEL|X1_9_ANSEL|X2_9_ANSEL|X2_5_ANSEL );
	else
		setup_adc_ports( X1_5_ANSEL|X2_9_ANSEL|X2_5_ANSEL );

	int16 adc_reading=0;

	if(!input(X2_3))	// Paddle button 1
		a=1;
	else
		a=0;
	if(!input(X2_4))	// Paddle button 2
		a = a+2;
  
	out_data_2[0]=a; //button state goes here
	
	set_adc_channel(X2_9_AN);	// POT_X
	delay_us(50);
	read_adc(ADC_START_ONLY);
	while(!(adc_done())) {    
	} 
	adc_reading = read_adc(ADC_READ_ONLY);
	pot2xf = (pot2xf*3+adc_reading)/4;
	adc_reading = (28560/pot2xf);
	adc_reading = adc_reading - 112;
	if (adc_reading>255)
		adc_reading = 255;
	out_data_2[1] = (int8)(127-adc_reading);

	set_adc_channel(X2_5_AN);	// POT_Y
	delay_us(50);
	read_adc(ADC_START_ONLY);
	while(!(adc_done())) {    
	} 
	adc_reading = read_adc(ADC_READ_ONLY);
	pot2yf = (pot2yf*3+adc_reading)/4;
	if (pot2yf<10)	// Paddles have been disconnected (or bad pot contact(!))
		mode2=1;
	adc_reading = (28560/pot2yf);
	adc_reading = adc_reading - 112;
	if (adc_reading>255)
		adc_reading = 255;
	out_data_2[2] = 127 - adc_reading;

}

void usb_megadrive_2_task(void) 
{
	#asm
	bsf	WPUB,3
	#endasm

	output_float(X2_7_ADD);
	
	if(!input(X2_6))  // Megadrive button B
		out_data_2[0] = 2;
	else
		out_data_2[0] = 0;
	if(!input(X2_9))  // Megadrive button C
		out_data_2[0] +=4;
	
	if(md_button_poll) {	
		output_low(X2_7); // 1st low
		delay_us(12);
	
		a=0;
		if(!input(X2_6))	// Megadrive button A
			a=1;
		
		if(!input(X2_9))  // Megadrive button Start
			a = a+8;
		output_high(X2_7);
		delay_us(12);
	
		output_low(X2_7); // 2nd low
		delay_us(12);        
		output_high(X2_7);
		delay_us(12);
		output_low(X2_7);	// 3rd low
		delay_us(12);
		if (input(X2_1) || input (X2_2))	// With 6-button controller 1/2/3/4 shold all be low
			mode2=3;			// Set mode to 3-button megadrive controller
		else
			mode2=4;
		output_high(X2_7);
		delay_us(12);
		if(!input(X2_1)&&mode2==4)  // Megadrive button Z
			a = a+64;
		if(!input(X2_2)&&mode2==4)  // Megadrive button Y
			a = a+32;
		if(!input(X2_3)&&mode2==4)  // Megadrive button X
			a = a+16;
		if(!input(X2_4)&&mode2==4)  // Megadrive button MODE
			a = a+128;
		mode2=3;
	
		output_low(X2_7);	// 4th low
		delay_us(12);        
		output_high(X2_7);
		delay_us(12);
		
	}
	out_data_2[0] = out_data_2[0]+a; //button state goes here

	if(!input(X2_2))			// down
		out_data_2[2]=126;
	else if(!input(X2_1))		// up
		out_data_2[2]=-126;
	else out_data_2[2] = 0;

	if(!input(X2_4))			// right
		out_data_2[1]=126;
    else if(!input(X2_3))		// left
		out_data_2[1]=-126;
	else out_data_2[1] = 0;
}

void main(void) 
{
	mode=3;		// Default is Megadrive
	mode2=3;	// Default is Megadrive
	int temp = 0;
	usb_init();
	PORT_E_PULLUPS(TRUE);	// Activates weak pull-up on pin RE3 (VPP/MCLR)
	PORT_B_PULLUPS(0b11000011);  // Activates weak pull-up on RB0,RB1,RB6,RB7

	setup_timer_1(T1_INTERNAL|T1_DIV_BY_1);	// Setup timer1 to use fclk/4 (12 MHz) as input

	// drive unused pins to GND:
	OUTPUT_LOW(UNUSED1);
	OUTPUT_LOW(UNUSED2);
	OUTPUT_LOW(UNUSED3);
	#ifdef UNUSED4
	OUTPUT_LOW(UNUSED4);
	#endif

	OUTPUT_HIGH(X1_5);		// Activates +5V to megadrive pin 5 port 1
	delay_ms(5);
	OUTPUT_FLOAT(X1_7_ADD);	// Tristate additional pin 7 since select carries no current
	OUTPUT_LOW(X1_7);		// Activates GND to megadrive pin 7 port 1(select)
	delay_us(15);
	if (INPUT(X1_3)||INPUT(X1_4)) {	// Check if either left or right is high
		mode = 1;	// There is not Megadrive controller attached in port 1
		OUTPUT_HIGH(X1_7);	// Activate +5V to pin 7 for Atari joy/paddles
		OUTPUT_HIGH(X1_7_ADD);	// Activate additional +5V to pin 7 for more current
		OUTPUT_FLOAT(X1_5);	// Do not drive pin 5 for Atari joy/paddles
		setup_adc_ports( X1_5_ANSEL|X2_9_ANSEL );  // AN11 for p1 POT_Y, AN12 for p1 POT_X, AN8 for p2 Y, AN9 for p2 X
		setup_adc(ADC_CLOCK_DIV_64);
	}	

	OUTPUT_HIGH(X2_5);		// Activates +5V to megadrive pin 5 port 2
	delay_ms(5);
	OUTPUT_FLOAT(X2_7_ADD);	// Tristate additional pin 7 since select carries no current
	OUTPUT_LOW(X2_7);		// Activates GND to megadrive pin 7 port 2(select)
	delay_us(15);
	if (INPUT(X2_3)||INPUT(X2_4)) {	// Check if either left or right is high
		mode2 = 1;	// There is not Megadrive controller attached in port 2
		OUTPUT_HIGH(X2_7);	// Activate +5V to pin 7 for Atari joy/paddles
		OUTPUT_HIGH(X2_7_ADD);	// Activate additional +5V to pin 7 for more current
		OUTPUT_FLOAT(X2_5);	// Do not drive pin 5 for Atari joy/paddles
		setup_adc_ports( X1_5_ANSEL|X2_5_ANSEL|X2_9_ANSEL );  // AN11 for p1 POT_Y, AN12 for p1 POT_X, AN8 for p2 Y, AN9 for p2 X
		setup_adc(ADC_CLOCK_DIV_64); // Only recommended ADC divisor for 48 MHz Fosc. Equals TAD period of 1.3 s
	}	

	while (TRUE) {
		if (usb_enumerated()) {
			
			if(TMR1IF) {				// enough time spent, megadrive buttons can be read
				set_timer1(MD_POLL_INT);
				md_button_poll=1;
				TMR1IF=0;
			} else
				md_button_poll=0;
				
			if (mode==1) {				// Check if paddles are connected (p1)
				set_adc_channel (X1_5_AN);	// POT_Y
				delay_us(20);
				temp = read_adc();
				if (temp > 10)		// If paddles are connected, input is higher than GND
					mode = 2;		// Because if not connected, resistor pulls to GND
			}

			if (mode == 1)
         		usb_joystick_1_task();
         	else if (mode == 2)
         		usb_paddle_task();
         	else
         		usb_megadrive_task();

			if (mode2==1) {				// Check if paddles are connected (p2)
				set_adc_channel (X2_5_AN);	// POT_Y
				delay_us(20);
				temp = read_adc();
				if (temp > 10)		// If paddles are connected, input is higher than GND
					mode2 = 2;		// Because if not connected, resistor pulls to GND
			}

			if (mode2 == 1)
         		usb_joystick_2_task();
         	else if (mode2 == 2)
         		usb_paddle_2_task();
         	else
         		usb_megadrive_2_task();

			while (!usb_put_packet(2,out_data_2,3,USB_DTS_TOGGLE))	// wait here for USB poll from host
			{
			}

		}
	}
}